---
title: Towards computer assisted modeling
theme: beige
revealOptions:
    transition: 'slide'
    slideNumber: true
---

<img style="height:350px" src=./images/openmole5.png>

## Reproducible Model Evaluation with OpenMOLE

#### Romain Reuillon // Mathieu Leclaire

---

## eX Modelo

<img src=./images/exmodelo.jpg>

<div style="margin-top: 40px">[http://exmodelo.org](exmodelo.org)</div>

---

## OpenMOLE Manifesto

 * Provide state-of-the-art exploration methods
 * Tackle scalability using distributed computing
 * Is not intrusive in the model logic
 * Make the modeling process (more) reproducible

---

<img src=./images/openmoleGalRaw.png>

---

## 1 - Model?
<img src="./images/modelIO.png"/> <br/><br/>

----

<img src=./images/blackbox.png>

----

<img src=./images/openmolefirefox.png>

----

## NetLogo on Fire

<pre style="brush: scala"><code>
val model =
  NetLogo6Task(
    workDirectory / "Fire.nlogo",
    List("setup", "while [any? turtles] [go]")) set (
    inputs += seed,
    outputs += (seed, density),
    netLogoInputs += (density, "density"),
    netLogoOutputs += ("burned-trees", burned)
  )
</code></pre>

----

## Rrrrrrrrrr

<pre style="brush: scala"><code>
val i = Val[Int]

val rTask =
  RTask("""
    source("function.R")
    function(i)""") set (
    resources += workDirectory / "function.R",
    rInputs += i
  )
</code></pre>

----

## Feeling snaky? Python

<pre style="brush: scala"><code>
val i = Val[Int]

val container =
  ContainerTask(
    "python:3.6-stretch",
    """python matrix.py data.csv ${i} out.csv""",
    install = Seq("pip install numpy")
  ) set (
    resources += workDirectory / "matrix.py",
    inputs += i
  )
</code></pre>

---

## 2 - Method?
      
<img src="./images/openmoleGalRaw.png"><br>

----

<br/>
<div style="text-align:center;">
  Data reconstruction<br/>
  Parameter estimation<br/>
  Sensitivity analysis<br/>
  Robustness assessment<br/>
  Optimisation<br/>
  ...<br/>
</div>


----

<div style="margin-top:40px">Designed in a <b>scalable</b> manner, handle <b>stochasticity</b>, usable on <b>any models and environments</b>.</div>

----

## Scripting

<pre style="brush: scala"><code>DirectSampling(
  evaluation = myModel,
  sampling =
    LHS(
      500 // number of points,
      diffusion in (10.0, 100.0),
      evaporation in (10.0, 100.0)
    )
  )
</code></pre>


----

<img src="./images/three_LHS.png"/>

----

### Limits

 * It produces a huge quantity of data.
 * It transforms a computation problem into a data-mining problem.
 * The parameter space is mostly left unexplored due to the curse of dimensionality.

----

### Reverse approach

<img style="height:400px" src=img/guided.jpg>

<p style="text-align:center">From *output* values to *parameter* values.</b>

----

### Exemple: Pattern search

<img style="height:500px" src=img/pse_algo.svg>

----

<img style="height:400px" src=img/hitmap.svg>

<p style="text-align:center;padding-top:10px">The inputs producing rare patterns have high fitness values.</p>

----

<img style="height:500px" src=img/flockingvolumediscovered.svg>

----

### Exemple OpenMOLE + SimPLU (IGN)

<video height="400px" autoplay controls>
  <source src="img/capture_visuPSE.webm" type="video/webm">
</video>

[simplu.openmole.org](http://simplu.openmole.org)

----

## PSE

<pre style="brush: scala;"><code>
val exploration =
  PSEEvolution(
    evaluation = modelTask,
    termination = 100,
    genome =
      Seq(
        param1 in (0.0, 1.0),
        param2 in (-10.0, 10.0)),
    objectives =
      Seq(
        output1 in (0.0 to 40.0 by 5.0),
        output2 in (0.0 to 4000.0 by 50.0))
  )
</code></pre>

---

## 3 - Execution environment?

<img src="./images/openmoleGalRaw.png">

----

### Naturally parallel processes

* Highly dimensional exploration 
* Replication of stochastics models
* Evolutionary algorithms

----

### Data-driven parallelism

<img src="./images/parallelism.jpg" style="width:65%">

----


### /!\ OpenMOLE is not MPI!

<br/>
<img src="./images/mpi.gif" style="width:65%">

----

### Local execution

<pre style="brush: scala"><code>DirectSampling(
  evaluation = myModel,
  sampling =   
    LHS(
      500,
      diffusion in (10.0, 100.0),
      evaporation in (10.0, 100.0)
    )
)
</code></pre>

----

### Distributed execution

<pre style="brush: scala"><code>val cluster = SLURMEnvironment("login", "cluster.domain.org")

DirectSampling(
  evaluation = myModel,
  sampling =   
    LHS(
      500,
      diffusion in (10.0, 100.0),
      evaporation in (10.0, 100.0)
    )
) on cluster
</code></pre>

----

### Supported environments today

 <br/>
<div style="text-align:center;">
 Multi-thread <br/>
 Delegation through SSH <br/>
 PBS <br/>
 SLURM <br/>
 Condor <br/>
 SGE <br/>
 OAR <br/>
 EGI Grid <br/>
 iExec (WIP) <br/>
</div>

---

## Reproducibility in OpenMOLE

<img src="./images/reproducible.jpg" style="width:75%">

----

## 1- Computation reproducibility 

<!--<img src="./images/mire.jpg" style="width:75%">-->

----

### Zero deployment approach
        
* No prior knowledge of remote environment needed
* No installation required on any machine
* User code and data are automatically deployed
* Easy to switch for one environment to another

----

## 2- Reproducibility of model qualities

---

### High-level language for model exploration

<pre style="brush: scala"><code>NSGA2Evolution(
  evaluation = model,
  genome = Seq(
    diffusion in (0.0, 99.0),
    evaporation in (0.0, 99.0)),
  objectives = Seq(deltaFood),
  termination = 1000
)
</code></pre>

----

## 3- Reproducibility of modeling process

<p style="text-align:center; margin-top: 50px">A general framework for computer assisted modeling.</p><br/>

Make the modeling process:
  * *traceable*: understand what choices were made,
  * *reproducible*: recompute the reason why they were made,
  * *reusable*: study alternative choices.

----

### Descriptive models

<p style="text-align:center">Design/evaluate a theory involving causal effects through its capacity to (re-)produce some patterns/data.</p><br/>
<img style="height:300px" src=img/descriptive_model.svg>

----

### Automate the confrontation of alternative mechanisms

<img style="height:400px; margin-top: 50px" src=img/model_familly.svg>

----

### Reproducible tiering of mechanisms

<img style="height:400px" src=img/familly_pareto.svg>

---

## Future tool fo reproducibility in OpenMOLE

<img src="./images/IPFS_logo.png" style="width:60%; margin-top: 50px">

<div style="text-align:center; margin-top: 50px">IPFS is the permanent web</div>
----

### Somewhere in the galaxy...

<pre style="font-size: 16px"><code>[reuillon:/tmp] $ ipfs add -r myexperiment/
added QmVpQqUFJ1cuJ14zGX8UA9qZx385vX5ycytzvc4TLSxSj3 myexperiment/Model.oms
added QmTkpGtwinafzVkFftGXwAKbZxJvMXnXoXsRCEP2nJx2v6 myexperiment/ants.nlogo
added QmaPNwYRBWktqxH7sqGy4FGt55neC4GB2yyQQ7FGLQu66d myexperiment/calibrate/Calibrate.oms
added QmdFjZpPSQAUxTqvs5omJMAWNPV59YtaKc9jNusi7BAYXP myexperiment/calibrate/Objective.oms
added Qmd6tFSeq4HbN3UrwUTurPvw6t9ZnUHP5WVsAkgVsS1d6f myexperiment/explore/Explore.oms
added QmPwBMnYQbaqSpdtHLy6891q6H347FnLH2NoAwNR3NLVaV myexperiment/pse/PSE.oms
added QmQFbKdS2kvkAGLMuMHkwmbcXeYTipAbg4cYD2X6RQoN4r myexperiment/replicate/Aggregate.oms
added QmZzJF8LDdW23DxbTxoDWeFDqSXzCTZtGALBxZfmjW7DFP myexperiment/replicate/Replicate.oms
added QmSwZdcEom1A7u3XD4YDeUUZTZpsMNTXNHucv664o2RHPF myexperiment/calibrate
added QmNwiUb63z3Mx7xdMk9UF3ZLRwVyxmXomUNMEFsrWGUufy myexperiment/explore
added QmXcoBC6Gimky2tktCxKa9tw3fqwBwLdjyHoUjHFHyQz3j myexperiment/pse
added QmXuuJBDp44Hnf6a3EovxyfToWw8AHit4s4pjZG9hfgNAq myexperiment/replicate
added QmaxahHruNUtCKeGEpmVUwBZhpdMr5KT9eYguUrQh485ip myexperiment
</pre></code>

<div style="font-size: 20px; margin-top: 40px">Publish: /ipfs/QmaxahHruNUtCKeGEpmVUwBZhpdMr5KT9eYguUrQh485ip</div>

----

### ... somewhere else in the galaxy

<pre style="font-size: 16px"><code>[chuffart:/tmp] $ ipfs ls /ipfs/QmaxahHruNUtCKeGEpmVUwBZhpdMr5KT9eYguUrQh485ip
QmVpQqUFJ1cuJ14zGX8UA9qZx385vX5ycytzvc4TLSxSj3 1122  Model.oms
QmTkpGtwinafzVkFftGXwAKbZxJvMXnXoXsRCEP2nJx2v6 19835 ants.nlogo
QmSwZdcEom1A7u3XD4YDeUUZTZpsMNTXNHucv664o2RHPF 1334  calibrate/
QmNwiUb63z3Mx7xdMk9UF3ZLRwVyxmXomUNMEFsrWGUufy 863   explore/
QmXcoBC6Gimky2tktCxKa9tw3fqwBwLdjyHoUjHFHyQz3j 641   pse/
QmXuuJBDp44Hnf6a3EovxyfToWw8AHit4s4pjZG9hfgNAq 886   replicate/
</pre></code>

<pre class="fragment" data-fragment-index="1" style="font-size: 16px; margin-top: 40px"><code>[chuffart:/tmp] $ ipfs cat QmZzJF8LDdW23DxbTxoDWeFDqSXzCTZtGALBxZfmjW7DFP
Replication (
  evaluation = ants hook ToStringHook(),
  seed = seed,
  replications = 10
)
</pre></code>

<pre class="fragment" data-fragment-index="2" style="font-size: 16px; margin-top: 40px"><code>[chuffart:/tmp] $ ipfs pin -r /ipfs/QmaxahHruNUtCKeGEpmVUwBZhpdMr5KT9eYguUrQh485ip</pre></code>

----


<img src="./images/filecoin-lifecycle.png" style="width:60%">

----

### OpenMOLE is on IPFS

<img src="./images/openmole-ipfs.png" style="width:80%; margin-bottom:20px">

<div style="font-size: 20px">OpenMOLE 8.0-RC5: /ipfs/QmSv3Rx1uM6LxQL6Aye5rxbAu8kxoNqtsrL7Eve3F4BThX</div>


---

## Want to use it?

* *Stable*: <a href="http://openmole.org" style="display:inline; font-size: 100%">openmole.org</a>
* *Dev*: <a href="http://next.openmole.org" style="display:inline; font-size: 100%">next.openmole.org</a>
* *User forum*: <a href="http://ask.openmole.org" style="display:inline; font-size: 100%">ask.openmole.org</a>
* *Source code*: <a href="http://github.com/openmole/openmole" style="display:inline; font-size: 100%">github.com/openmole/openmole</a>
* *Demo instance*: <a href="http://demo.openmole.org" style="display:inline; font-size: 100%">demo.openmole.org</a>
* *Grid accesss*: <a href="http://iscpif.fr/services" style="display:inline; font-size: 100%">iscpif.fr/services</a>
* *eX Modelo*: <a href="http://exmodelo.org" style="display:inline; font-size: 100%">exmodelo.org</a>

---

## Hands on !

<img src="./images/mole.jpg" style="width:80%; margin-bottom:20px">


